#!/usr/bin/env python

############################################################################
#    Copyright (C) 2006 by Hasan Abdel Halim                               #
#    abdelhal@informatik.uni-bonn.de                                       #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                          #
#    Date : %date% at %time%                                               #
#    Filename : MainWindow.py                                                 # 
############################################################################

from qt import *
from FormMain import *
from ObexftpXML import *
from OperationsManager import *
from ObexftpInterface import *
from Copy import *
from Paste import *
from Delete import *
from Move import *
from OperationInfo import *

class MainWindow (QMainWindow):

    """
        QObexftp's Main Window 
        :version:
        :author:
    """
    def __init__(self, parent = None, name = None, fl = 0):
        QMainWindow.__init__(self, parent, name, fl)
      
        self.manager = OperationsManager( "QObexftp Operations Manager", self )
        self.mobile = ObexftpInterface()
      
        self.manager.setEngine( self.mobile )
 
        if not name:
            self.setName("QObexftp Main Window")
        
      #TODO: seperate the initilaization in seperate function
        if not self.mobile.setup_pcsuite():
            print "Failed to initialize engine"
            return 
      
        self.setup()
      
        self.connect(  self.main_form.lstLocal,
                            SIGNAL("doubleClicked(QListViewItem*)"),
                            self.change_local_dir )
        self.connect(  self.main_form.lstMobile,
                            SIGNAL("doubleClicked(QListViewItem*)"),
                            self.change_mobile_dir )
        self.connect(  self.main_form.lstMobile,
                            SIGNAL("contextMenuRequested(QListViewItem*,const QPoint &,int)"),
                            self.showMobileContextMenu )
        self.connect(  self.main_form.lstLocal,
                            SIGNAL("contextMenuRequested(QListViewItem*,const QPoint &,int)"),
                            self.showLocalContextMenu )
        self.connect( self.main_form.btnFrmMob,
                            SIGNAL("clicked()"),
                            self.download_file)
        self.connect( self.main_form.btnToMob,
                            SIGNAL("clicked()"),
                            self.upload_file)
        self.connect( self.actions["home"],
                            SIGNAL("activated()"),
                            self.homeClicked)
        self.connect( self.actions["refresh"], 
                            SIGNAL("activated()"),
                            self.refreshClicked)
        self.connect( self.actions["stop"], 
                            SIGNAL("activated()"),
                            self.stopClicked)
        self.connect( self.actions["remove"], 
                            SIGNAL("activated()"), 
                            self.removeClicked)
        self.connect( self.actions["copy"], 
                            SIGNAL("activated()"), 
                            self.copyClicked) 
        self.connect( self.actions["move"], 
                            SIGNAL("activated()"), 
                            self.moveClicked )   
        self.connect( self.actions["paste"], 
                            SIGNAL("activated()"), 
                            self.pasteClicked)    
    
    def setup(self):
        """ 
            setup method to initalize main widgets of the main window   
        """
        self.main_form = FormMain(self,"QObexftp")
        self.setCentralWidget(self.main_form)
        self.createPixmaps()
        self.createActions()
        self.createToolBar()
        self.createContextMenu()
        self.createProgressBar()
        self.initMainForm()
          
        # initalize global variables
        self.operations_no = 0
        self.done_no = 0
        self.current_dirs = {}
        self.current_dirs["local"] = QDir.homeDirPath()
        self.current_dirs["mobile"] = ''
        self.refreshClicked()

    
    def createPixmaps(self):
        """
            Creates main window pixmaps.
        """
        self.pixs = {}
        self.pixs["local"] = QPixmap.fromMimeSource("pix/computer.png")
        self.pixs["mobile"] = QPixmap.fromMimeSource("pix/mobile.png")
        self.pixs["file"] = QPixmap.fromMimeSource("pix/file.png")
        self.pixs["folder"] = QPixmap.fromMimeSource("pix/folder.png")
        self.pixs["frm-mobile"] = QPixmap.fromMimeSource("pix/frm-mobile.png")
        self.pixs["to-mobile"] = QPixmap.fromMimeSource("pix/to-mobile.png")
        self.pixs["refresh"] = QPixmap.fromMimeSource("pix/reload.png")
        self.pixs["home"] = QPixmap.fromMimeSource("pix/home.png")
        self.pixs["paste"] = QPixmap.fromMimeSource("pix/paste.png")
        self.pixs["remove"] = QPixmap.fromMimeSource("pix/remove.png")
        self.pixs["new-folder"] = QPixmap.fromMimeSource("pix/folder-new.png")
        self.pixs["copy"] = QPixmap.fromMimeSource("pix/copy.png")
        self.pixs["move"] = QPixmap.fromMimeSource("pix/cut.png")
        self.pixs["stop"] = QPixmap.fromMimeSource("pix/stop.png")
  
  
    def createActions( self ):
        """ 
            Creates main window actions .
        """
        self.actions = {}
        self.actions["refresh"] = QAction(self, "Refresh")
        self.actions["refresh"].setText("Refresh")
        self.actions["refresh"].setIconSet( QIconSet( self.pixs["refresh"] ) )
        
        self.actions["stop"] = QAction(self, "Stop")
        self.actions["stop"].setText("Stop")
        self.actions["stop"].setIconSet( QIconSet( self.pixs["stop"] ) )
        
        self.actions["home"] = QAction( self, "Home" )
        self.actions["home"].setText("Home")
        self.actions["home"].setIconSet ( QIconSet ( self.pixs["home"] ) )
        
        self.actions["remove"] = QAction(self, "Remove")
        self.actions["remove"].setText("Delete")
        self.actions["remove"].setIconSet( QIconSet( self.pixs["remove"] ) )
        
        self.actions["copy"] = QAction( self, "Copy" )
        self.actions["copy"].setText("Copy")
        self.actions["copy"].setIconSet( QIconSet( self.pixs["copy"] ) )

        self.actions["move"] = QAction( self, "Move" )
        self.actions["move"].setText("Move")
        self.actions["move"].setIconSet( QIconSet( self.pixs["move"] ) )

        self.actions["paste"] = QAction( self, "Paste" )
        self.actions["paste"].setText("Paste")
        self.actions["paste"].setIconSet( QIconSet( self.pixs["paste"] ) )
        self.actions["paste"].setEnabled(0)
        
        self.actions["new-folder"] = QAction( self, "New Folder ")
        self.actions["new-folder"].setText("New Folder")
        self.actions["new-folder"].setIconSet( QIconSet( self.pixs["new-folder"] ) )

    
    def createToolBar( self ):
        """ 
            Creates the main toolbar for the main window 
        """
        self.toolbars = {}
        self.toolbars["main"] = QToolBar("Main", self )
        self.actions["home"].addTo( self.toolbars["main"] )
        self.actions["refresh"].addTo( self.toolbars["main"] )
        self.actions["stop"].addTo( self.toolbars["main"] )
        
        self.toolbars["edit"]  = QToolBar("Edit", self)
        self.actions["paste"].addTo( self.toolbars["edit"] )
        self.actions["remove"].addTo( self.toolbars["edit"] )
        self.actions["new-folder"].addTo( self.toolbars["edit"] )
    
    
    def createContextMenu( self ):
        """
            Creates context menu of the main applicaiton 
        """
        self.contextMenus = {}
        self.contextMenus["mobile"] = QPopupMenu(self)
        self.actions["copy"].addTo( self.contextMenus["mobile"] )
        self.actions["move"].addTo( self.contextMenus["mobile"] )
        self.actions["remove"].addTo( self.contextMenus["mobile"] )
        self.contextMenus["mobile"].insertSeparator()
        self.actions["refresh"].addTo( self.contextMenus["mobile"] )
        
        self.contextMenus["local"] = QPopupMenu(self)
        self.actions["paste"].addTo( self.contextMenus["local"] )
        self.contextMenus["local"].insertSeparator()
        self.actions["home"].addTo( self.contextMenus["local"] )
        self.actions["refresh"].addTo( self.contextMenus["local"] )

    
    def createProgressBar( self ):
        """
            Creates progress bar of the main window 
        """
        self.progress_bar = QProgressBar()
        self.statusBar().addWidget ( self.progress_bar )
        self.progress_bar.hide()
        self.progress_bar.setMaximumHeight(16)

    
    def initMainForm( self ):
        """
            Initalizes the main form of the main window.
        """
        self.main_form.lbl_local_pix.setPixmap( self.pixs["local"]  )
        self.main_form.lbl_mobile_pix.setPixmap( self.pixs["mobile"]  )
        
        self.main_form.btnFrmMob.setIconSet( QIconSet( self.pixs["frm-mobile"] ) )
        self.main_form.btnToMob.setIconSet( QIconSet(  self.pixs["to-mobile"] ) )
        
        self.main_form.lstMobile.addColumn("Name")
        self.main_form.lstMobile.addColumn("Label")
        self.main_form.lstMobile.addColumn("Memory Type")
        self.main_form.lstMobile.addColumn("Permissions")        
        self.main_form.lstMobile.setSelectionMode( QListView.Extended )
     
        self.main_form.lstLocal.addColumn("Name")
        self.main_form.lstLocal.addColumn("Size")
        self.main_form.lstLocal.addColumn("Modified")
        self.main_form.lstLocal.addColumn("Permissions")
        self.main_form.lstLocal.setSelectionMode( QListView.Extended )

    
    def __del__(self):
        """
            Default constructor 
        """
        if self.manager.running():
            self.manager.stop()
            self.manager.wait()
    
    
    def customEvent(self, event):
        """
            Synchronization between operations manager thread and main window thread.
        """
      #TODO: Create special class for events 
        if event.type() == 333333:
            # start event of operations 
            self.done_no = 0
            self.operations_no = 0
            self.progress_bar.show()

        if event.type() == 555555:
            # start event of operations 
            self.progress_bar.hide()
            self.displayOnStatusBar("Operations were canceled by user ", 2000)
        
        if event.type() == 100000:
            # progress event of operations 
            self.done_no = 1
            self.operations_no = int ( event.data() )
    
        if event.type() == 222222:
            # finish event of operations 
            self.progress_bar.hide()
            self.refreshClicked()
        
        self.progress_bar.setTotalSteps( self.operations_no )
        self.progress_bar.setProgress( self.done_no )
    
    
    def update_local_list(self):
        """
            Refreshes the list of local files to the current_dirs["local"]. 
        """
        self.main_form.lbl_local.setText( self.current_dirs["local"] )
        
        dir = QDir(self.current_dirs["local"])
       
        self.main_form.lstLocal.clear()
        #TODO: fix the sorting issue, make directories before files 
        files = dir.entryList( QDir.All, QDir.DirsFirst )
     
        for file in files:
            item = QListViewItem( self.main_form.lstLocal, file )
            info = QFileInfo( self.current_dirs["local"] +'/'+ file )
            if info.isFile():
                item.setPixmap( 0, self.pixs["file"] )
            else:
                item.setPixmap( 0, self.pixs["folder"] )
 
 
    def update_mobile_list(self):
        """
            Refreshes the list of mobile files to the current_dirs["mobile"].
        """
        # TODO: make cache for mobile directories and only update on changes like download, delete
        #   etc or based on time.
        self.main_form.lbl_mobile.setText( self.current_dirs["mobile"]  )
        
        xml = self.mobile.dir_list(self.current_dirs["mobile"])
        dir = parse_obex_xml (xml)
        folders = dir['folders']
        files = dir['files']

        self.main_form.lstMobile.clear()
        # TODO: check the type before assuming it
        item = QListViewItem(self.main_form.lstMobile,".","","DEV")
        item.setPixmap( 0, self.pixs["folder"] )
        item = QListViewItem(self.main_form.lstMobile,"..","","DEV")
        item.setPixmap( 0, self.pixs["folder"] )
        
        for folder in folders:
            item = QListViewItem(
                        self.main_form.lstMobile,
                        folder['Name'],
                        folder['Label'],
                        folder['MEM-Type'],
                        folder['User-Perm']
                        )
            item.setPixmap( 0, self.pixs["folder"] )
    
        for file in files:
            item = QListViewItem(
                        self.main_form.lstMobile,
                        file['Name'],
                        file['Size'],
                        file['Modified'],
                        file['User-Perm']
                        )
            item.setPixmap(0, self.pixs["file"] )

    
    def change_mobile_dir( self, dir_name):
        """
            Changes the current mobile directory to dir_name.
        """
        # don't change if it is not a directory (Type = MMC, DEV ).
        if dir_name.text(2) != "MMC" and dir_name.text(2) != "DEV":
                return 
        
        if dir_name.text(0) == '.':
            dir_name = self.current_dirs["mobile"]
        elif dir_name.text(0) == '..':
            dir_name = str(self.current_dirs["mobile"]).rsplit('\\', 1)[0]
            # fixes drives bug
            if str(self.current_dirs["mobile"]).endswith(':'):
                dir_name = ''
        else:
            if  self.current_dirs["mobile"] == '':
                dir_name = dir_name.text(0)
            else:
                dir_name = self.current_dirs["mobile"] + "\\" + dir_name.text(0)
        
        if not dir_name:
            dir_name = ''
        
        self.current_dirs["mobile"] = dir_name
        self.update_mobile_list()

    
    def change_local_dir(self, dir_name):
        """
            Changes the current mobile directory to dir_name.
        """
        if dir_name.text(0) == '.':
            dir_name = self.current_dirs["local"]
        elif dir_name.text(0) == '..':
            dir_name = str(self.current_dirs["local"]).rsplit('/', 1)[0]
        else:
            dir_name = self.current_dirs["local"] + "/" + dir_name.text(0)
        
        if not dir_name:
            dir_name = '/'
        
        info = QFileInfo( dir_name )
        
        # checks if the selection is file, updates only on directories.
        if info.isFile():
            return 
        
        # checks if the directory is readable, otherwise don't change the current path.
        if not info.isReadable():
            self.displayOnStatusBar("Access denied", 1000)
            return
        
        self.current_dirs["local"] = dir_name
        self.update_local_list()

    def isMobileFile(self, file_name, file_type ):
        if not file_name or not file_type:
            return 0
        
        if file_name == "." or file_name == "..":
            return 0
        
        if file_type == "DEV" or file_type == "MMC":
            return 0
        #TODO: make more powerfull checking for the rest of types
        return 1

    
    def getSelectedLocalfiles(self):
        selected_files =[]
        
        it = QListViewItemIterator( self.main_form.lstLocal )
        itm = it.current()
        while itm:
            if itm.isSelected():
                info = QFileInfo( self.current_dirs["local"]+'/'+ itm.text(0) )
                file_name = itm.text(0)
                if info.isFile():   # only multiple selection of files is allowed 
                    selected_files.append( file_name )
            it +=1
            itm = it.current()
        
        return selected_files
    
    
    def getSelectedMobilefiles(self):
        selected_files = []
        
        it = QListViewItemIterator( self.main_form.lstMobile )
        itm = it.current()
        while itm:
            if itm.isSelected():
                file_name = itm.text(0)
                file_type = itm.text(2)
                if self.isMobileFile( file_name,  file_type ):
                    selected_files.append( file_name )
            it +=1
            itm = it.current()
        
        return selected_files

    
    def download_file(self):
        """
            Downloads the selected file from mobile to current_dirs["local"].
        """
        #TODO: clear selections of folders in clicked event 
        for file in self.getSelectedMobilefiles():
            dwn = Copy ( OperationInfo( file, 
                                                    self.current_dirs["mobile"],
                                                    self.current_dirs["local"]) )
            self.manager.add(dwn)


    
    def upload_file(self):
        """
            Uploads the selected file to current_dirs["mobile"].
        """
        if self.current_dirs["mobile"] == '':
            self.displayOnStatusBar("Uploading to root directory is not allowed", 1000)
            return
        
        for file in self.getSelectedLocalfiles():
            up = Paste ( OperationInfo( file, 
                                                    self.current_dirs["local"],
                                                    self.current_dirs["mobile"]) )
            self.manager.add(up)

    
    def remove_file(self):
        """
            removes the current selected file from mobile.
        """
        for file in self.getSelectedMobilefiles():
            dl = Delete ( OperationInfo( file, self.current_dirs["mobile"] ) )
            self.manager.add(dl)


    def stopClicked( self ):
        self.manager.stop()
    
    def refreshClicked( self ):
        """
            Refreshes both list the local, and mobile ones.
        """
        self.update_mobile_list()
        self.update_local_list()       

    def homeClicked( self ):
        """
            Home action has been activated.
        """
        self.current_dirs["local"] = QDir.homeDirPath()
        self.update_local_list()
    
    def removeClicked( self ):
        """
            Remove action has been activated.
        """
        self.remove_file()


    def copyClicked( self ):
        self.copied={}
        self.copied["dir"] = self.current_dirs["mobile"]
        self.copied["files"] = self.getSelectedMobilefiles()
        self.actions["paste"].setEnabled(1)
    
 
    def moveClicked( self ):
        self.copied={} # shall I do this ?
        self.moved={}
        self.moved["dir"] = self.current_dirs["mobile"]
        self.moved["files"] = self.getSelectedMobilefiles()
        self.actions["paste"].setEnabled(1)
    
    
    def pasteClicked(self ):
        if self.copied.has_key("files"):
            for file in self.copied["files"]:
                dwn = Copy ( OperationInfo( file, 
                                                        self.copied["dir"],
                                                        self.current_dirs["local"]) )
                self.manager.add(dwn)

    #TODO : ensure only one operation is done either move or copy 
        if self.moved.has_key("files"):
            for file in self.moved["files"]:
                mv = Move ( OperationInfo( file, 
                                                        self.moved["dir"],
                                                        self.current_dirs["local"]) )
                self.manager.add(mv)

        self.copied={} # shall I reset the copy selection 
        self.moved={} # shall I reset the copy selection 
        self.actions["paste"].setEnabled(0)
    
    
    def showMobileContextMenu( self, item, pos ):
        """
            Shows context menu on right click event 
        """
        self.contextMenus["mobile"].exec_loop( pos )

    
    def showLocalContextMenu( self, item, pos ):
        """
            Shows context menu on right click event 
        """
        self.contextMenus["local"].exec_loop( pos )

    
    def displayOnStatusBar(self, msg, time = 1000 ):
        """
            Displays a message in the status Bar with a givin time.
        """
        self.statusBar().message( msg, time )
