#!/usr/bin/env python

############################################################################
#    Copyright (C) 2006 by Hasan Abdel Halim                               #
#    abdelhal@informatik.uni-bonn.de                                       #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                          #
#    Date : %date% at %time%                                               #
#    Filename : ObexftpInterface.py                                                 # 
############################################################################

import sys
import os.path
from subprocess import *

class ObexftpInterface:

  """
  
  :version:
  :author:
  """

  def __init__(self):
      self.obexftp_cmd= "/usr/bin/obexftp"
      self.setup()
      self.default_if = None

  def setup(self):
    args = [self.obexftp_cmd, '-u']
    output = self.process( args )
    output = output.split('\n')
    self.find_interfaces( output )

  def find_interfaces(self, output):
    #    Splits the output to 
    #    1. number of interfaces
    #    2. description of these interfaces
    ifs_no = output[0].split(None,2)[1]
    ifs_desc = output[1:]
    
    #    Extracts interfaces list with the following format:
    #    ['Interface 0:', '\tManufacturer: Nokia', '\tProduct: Nokia N70',
    #    '\tInterface description: SYNCML-SYNC']
    #    ['Interface 1:', '\tManufacturer: Nokia', '\tProduct: Nokia N70',
    #    '\tInterface description: PC Suite Services']
    self.interfaces = []
    for i in range( int(ifs_no) ):
        step = 4 * i
        interface = {
            'Number': int ( 
                ifs_desc[step].split('Interface ',1)[1].replace(':','') ),
            'Manufacturer': 
                ifs_desc[step+1].split(':',1)[1].replace('\n','').strip(),
            'Product': 
                ifs_desc[step+2].split(':',1)[1].replace('\n','').strip(),
            'Description': 
                ifs_desc[step+3].split(':',1)[1].replace('\n','').strip()
        }
        self.interfaces.append( interface )

  # returns current usb interfaces
  def get_interfaces(self):
    return self.interfaces
  
  # sets the default interface which is used for 
  # file operations like ( upload, download, etc, ..).
  def set_default_interface(self, interface_no):
    if  self.interfaces[ interface_no ]:
        self.default_if = interface_no
        print "Default interface was set to ", interface_no
    else:
        print "Failed to setup default interface:"
        print "\t", interface_no, " is Not defined." 
    
  # returns the current interface which is used for
  # file operations.
  def get_default_interface(self):
      return self.default_if

  def setup_pcsuite(self):
      interfaces = self.get_interfaces()
    
    # finds PC SUITE interface, then sets it as default interface
      for interface in interfaces:
        if interface['Description'].find('PC Suite') is not -1:
            self.set_default_interface( interface['Number'] )
            break
        
        # fails if there is no default interface
      if self.get_default_interface() is None:
        print " Failed to find suitable PC SUITE interface "
        return 0
        
      return 1


  def download(self, file_name, src_dir, dest_dir, move='false'):
    """
    

    @return  :
    @author
    """
    args = [self.obexftp_cmd] 
    args.extend(['-u', '%s' % self.default_if])
    
    if src_dir != "" and file_name != "":
        args.extend( ['-c', '%s' % src_dir] )
        
        if move == 'true' :
            args.extend(['-G'])
        else:
            args.extend( ['-g'] )
        
        args.extend( ['%s' % file_name] )

    self.process( args ) 
    
    if dest_dir != "":
        self.move_to_dest( file_name, dest_dir)

  def upload(self, file_name, src_dir, dest_dir):
    """
    

    @return  :
    @author
    """
    args = [self.obexftp_cmd]
    args.extend(['-u', '%s' % self.default_if])
    
    if dest_dir == "" or file_name == "":
        print "failed to upload"
        return  
    
    args.extend(['-c', '%s' % dest_dir])
    
    if src_dir != "":
        file_name = src_dir + '/' + file_name
    
    args.extend(['-p', '%s' % file_name])

    self.process( args )


  def move(self):
    """
    

    @return  :
    @author
    """
    pass

  def delete(self, file_name, src_dir):
    """
    

    @return  :
    @author
    """
    args = [self.obexftp_cmd] 
    args.extend(['-u', '%s' % self.default_if])
    
    if file_name != "" or src_dir != "":
        args.extend(['-k', '%s\\%s' % (src_dir, file_name) ])

    self.process( args ) 

  def capability(self):
    """
    

    @return  :
    @author
    """
    args = [self.obexftp_cmd, '-X']
    xml = self.process( args )
    return xml

  def mkdir(self):
    """
    

    @return  :
    @author
    """
    pass

  def dir_list(self, dir_name):
    """
    

    @return  :
    @author
    """
    args = [self.obexftp_cmd]  
    args.extend(['-u', '%s' % self.default_if])
    
    if dir_name is not "":
        args.extend(['-c', '%s' % dir_name])
    
    args.extend(['-l'])
    
    xml = self.process( args )
    return xml

  def process( self, args ):
      print "calling process ", args 
      pid = Popen( args, stdout=PIPE,
                         stderr=PIPE )
      output = pid.stdout.read()
      
      print "Output", pid.stdout.readlines()
      print "Errors ", pid.stderr.readlines()
      
      return output

  def move_to_dest(self, file_name, dest_dir):
      Popen(['mv','%s' % file_name, '%s' % dest_dir])
